# -*- coding: utf-8 -*-

import json
from typing import Dict, List, Any, Union

import pandas as pd

# from .country_code_profile import CountryCodeProfile
# from .profile import ChannelIdTypeProfile


class BaseProfile():

    def __init__(self) -> None:
        self.dim :int = 0

    def convert(self, value):
        raise NotImplemented

    def inconvert(self, embedding: List[Union[int, float]]) -> str:
        raise NotImplemented


class CountryCodeProfile(BaseProfile):
    def __init__(self) -> None:
        super().__init__()
        self.dim = 3

    def convert(self, value):
        try:
            value = int(value)
        except Exception as e:
            return [0, 0, 1]
        if value == 86:
            return [1, 0, 0]
        else:
            return [0, 1, 0]

    def inconvert(self, embedding):
        if embedding[0] == 1:
            return 'china'
        elif embedding[1] == 2:
            return 'abroad'
        else:
            return 'unknown'


class ChannelIdTypeProfile(BaseProfile):
    def __init__(self) -> None:
        super().__init__()
        self.dim = 3

    def convert(self, value):
        try:
            value = int(value)
        except Exception as e:
            return [0, 0, 1]
            
        if value == 1:
            return [1, 0, 0]
        elif value == 2:
            return [0, 1, 0]
        else:
            return [0, 0, 1]

    def inconvert(self, embedding):
        if embedding[0] == 1:
            return 'ios'
        elif embedding[1] == 2:
            return 'android'
        else:
            return 'other'


class FfromLoginProfile(BaseProfile):
    """
    登录来源
        主要是不同android品牌特征，android和ios特征在其他画像中已有，这里不重复构建
    """
    def __init__(self) -> None:
        super().__init__()
        self.brand_list = ['huawei', 'vivo', 'oppo', 'xiaomi']
        self.dim = len(self.brand_list) + 1

    def convert(self, value):
        ret = [0, 0, 0, 0, 0]
        try:
            value = value.lower()
        except Exception as e:
            return ret

        for i, v in enumerate(self.brand_list):
            if v in value:
                ret[i] = 1
                break
        else:
            ret[self.dim - 1] = 1
        return ret

    def inconvert(self, embedding):
        for emb, name in zip(embedding, self.brand_list):
            if emb == 1:
                return name
        return 'other'


class UserPreferenceCateProfile(BaseProfile):

    def __init__(self) -> None:
        super().__init__()
        self.dim = 8

        self.cate_list = ['个人成长', '亲子教育', '人际关系', '婚姻家庭', '心理健康', '恋爱情感', '情绪压力', '职场发展']
        self.cate_index = {
            cate: index for index, cate in enumerate(self.cate_list)
        }

    def convert(self, value):
        ret = [0.] * 8
        if pd.isnull(value):
            return ret
        
        if isinstance(value, str):
            try:
                value = json.loads(value)
            except Exception as e:
                return ret

        for info in value:
            ret[self.cate_index[info['cate_name']]] = info['preference_score']

        return ret

    def inconvert(self, embedding):
        ret = {}
        for emb, name in zip(embedding, self.cate_list):
            ret[name] = emb
        return json.dumps(ret, ensure_ascii=False)


class NumClassProfile(BaseProfile):

    def __init__(self, split_points, mode='le') -> None:
        """
        mode : le 前开后闭； be: 前闭后开；默认前开后闭
        """
        super().__init__()
        self.dim = len(split_points) + 1
        self.split_points = split_points
        self.mode = mode

    def value_index(self, value):
        for i, v in enumerate(self.split_points):
            if self.mode == 'be':
                if value < v:
                    return i
            else:
                if value <= v:
                    return i
        return len(self.split_points)

    def convert(self, value):
        ret = [0] * (len(self.split_points) + 1)
        if pd.isnull(value):
            return ret
        try:
            value = float(value)
            index = self.value_index(value)
            ret[index] = 1
        except:
            return ret
        return ret


    def inconvert(self, embedding):
        ret = ''
        # 确保embedding中有包含1的值
        if embedding.count(1) == 0:
            return ''

        index = embedding.index(1)
        if index == 0:
            if self.mode == 'be':
                ret = '< {}'.format(self.split_points[0])
            else:
                ret = '<= {}'.format(self.split_points[0])
        elif index == (self.dim - 1):
            if self.mode == 'be':
                ret = '>= {}'.format(self.split_points[-1])
            else:
                ret = '> {}'.format(self.split_points[-1])
        else:
            if self.mode == 'be':
                ret = '{} < {}'.format(self.split_points[index-1], self.split_points[index])
            else:
                ret = '{} <= {}'.format(self.split_points[index-1], self.split_points[index])
        return ret


class AidiCstBiasPriceProfile(BaseProfile):
    """ 用户偏好价格 """

    def __init__(self) -> None:
        super().__init__()
        self.dim = 6
        self.price_groups = [
            '[0, 50)',
            '[50, 100)',
            '[100, 200)',
            '[200, 500)',
            '[500, 1000)',
            '[1000, )',
        ]

    def convert(self, value):
        ret = [0] * 6
        if pd.isnull(value):
            return ret

        json_object = json.loads(value)

        for v in json_object:
            try:
                ret[v['level'] - 1] = 1
            except Exception:
                pass
        return ret

    def inconvert(self, embedding):
        ret = ''
        # 确保embedding中有包含1的值
        if embedding.count(1) == 0:
            return ''

        index = embedding.index(1)
        ret = self.price_groups[index]
        return ret


class MultiChoiceProfile(BaseProfile):

    def __init__(self, option_dict: Dict[Any, int]) -> None:
        super().__init__()
        self.dim = 6
        self.option_dict = option_dict
        self.re_option_dict = {v: k for k, v in self.option_dict.items()}


    def convert(self, value: List):
        ret = [0] * len(self.option_dict)
        if pd.isnull(value):
            return ret

        value = json.loads(value)

        for v in value:
            try:
                i = self.option_dict[v]
                ret[i] = 1
            except Exception as e:
                pass
        return ret


    def inconvert(self, embedding):
        ret = []

        for index, emb in enumerate(embedding):
            if emb == 1:
                ret.append(self.re_option_dict.get(index, ''))

        return '[{}]'.format(', '.join(map(str, ret)))


class CityProfile(BaseProfile):
    """ 基于邮编的城市编码 """

    def __init__(self, level=2) -> None:
        """
            level: 级别，2-省/直辖市 ； 3-区； 4-区；6-投递区
        """
        super().__init__()

        self.level = level
        self.dim = self.level * 10


    def convert(self, value):
        ret = [0] * self.dim

        if pd.isnull(value):
            return ret

        value = str(value)
        try:
            for i, _n in enumerate(value[:self.level]):
                n = int(_n)
                ret[i * 10 + n] = 1

        except Exception as e:
            pass
        return ret

    
    def inconvert(self, embedding):
        # 邮编固定都是6
        ret = [0] * 6
        for index, _emb in enumerate(embedding):
            emb = int(_emb)
            if emb == 1:
                ret[int(index / 10)] = index % 10
        return ''.join(map(str, ret))


class AidiCstBiasCityProfile(CityProfile):
    """ 支持多个城市编码 """

    def __init__(self, level=2) -> None:
        super().__init__(level=level)


    def convert(self, value_object):
        ret = [0] * self.dim

        if pd.isnull(value_object):
            return ret

        if not value_object:
            return ret

        if isinstance(value_object, str):
            try:
                value_object = json.loads(value_object)
            except Exception as e:
                pass

        if isinstance(value_object, dict):
            for value in value_object.get('in', []):
                try:
                    for i, _n in enumerate(value[:self.level]):
                        n = int(_n)
                        ret[i * 10 + n] = 1
                except Exception as e:
                    pass
        return ret


profile_converters = [
    ['country_code', CountryCodeProfile()],
    ['channel_id_type', ChannelIdTypeProfile()],
    ['ffrom_login', FfromLoginProfile()],
    ['user_preference_cate', UserPreferenceCateProfile()],
    ['consult_pay_money', NumClassProfile([100, 300, 1000])],
    ['listen_pay_money', NumClassProfile([0, 50])],
    ['test_items_pay_money', NumClassProfile([0, 10])],
    ['course_pay_money', NumClassProfile([0])],
    ['consult_order_num', NumClassProfile([0, 1])],
    ['listen_order_num', NumClassProfile([0])],
    ['test_items_order_num', NumClassProfile([0])],
    ['course_order_num', NumClassProfile([0])],
    ['aidi_cst_bias_city', AidiCstBiasCityProfile()],
    ['aidi_cst_bias_sex', MultiChoiceProfile({i + 1: i for i in range(2)})],
    ['aidi_cst_bias_price', AidiCstBiasPriceProfile()],
    ['aidi_cst_bias_server_type', MultiChoiceProfile({i + 1: i for i in range(4)})],
    ['user_login_city', CityProfile()],
    ['d30_inquire_order_num', NumClassProfile([0, 1])],
    ['d30_session_num', NumClassProfile([0, 1])],
]

