# -*- coding: utf-8 -*-

import os
import json
from typing import List, Dict

import faiss
import numpy as np

from ydl_ai_recommender.src.core.profile_manager import ProfileManager
from ydl_ai_recommender.src.data.mysql_client import MySQLClient
from ydl_ai_recommender.src.utils import get_conf_path, get_data_path


class Recommender():

    def __init__(self) -> None:
        pass

    def recommend(self, user) -> List:
        raise NotImplemented


class UserCFRecommender(Recommender):

    def __init__(self, top_n=5, k=5, is_lazy=True) -> None:
        super().__init__()
        # 召回 top_n 个相似用户
        self.top_n = top_n
        # 每个召回的用户取 k 个相关咨询师
        self.k = k
        if is_lazy is False:
            self.client = MySQLClient.create_from_config_file(get_conf_path())
        self.manager = ProfileManager()
        self.local_file_dir = get_data_path()
        self.load_data()


    def load_data(self):
        order_user_embedding = []
        order_user_ids = []
        order_user_counselor_index = {}
        default_counselor = []

        with open(os.path.join(self.local_file_dir, 'user_embeddings_ids.txt'), 'r', encoding='utf-8') as f:
            order_user_ids = [line.strip() for line in f]

        with open(os.path.join(self.local_file_dir, 'user_embeddings.json'), 'r', encoding='utf-8') as f:
            order_user_embedding = json.load(f)

        with open(os.path.join(self.local_file_dir, 'user_doctor_index.json'), encoding='utf-8') as f:
            order_user_counselor_index = json.load(f)

        with open(os.path.join(self.local_file_dir, 'top50_supplier.txt'), 'r', encoding='utf-8') as f:
            default_counselor = [line.strip() for line in f]

        self.order_user_embedding = order_user_embedding
        self.order_user_ids = order_user_ids
        self.order_user_counselor_index = order_user_counselor_index
        self.default_counselor = [{
            'counselor': str(user),
            'score': index + 1,
            'from': 'top_50',
        } for index, user in enumerate(default_counselor)]

        self.index = faiss.IndexFlatL2(len(self.order_user_embedding[0]))
        self.index.add(np.array(self.order_user_embedding))


    def get_user_profile(self, user_id):
        sql = 'SELECT * FROM ads.ads_register_user_profiles'
        sql += ' WHERE uid={}'.format(user_id)
        try:
            _, all_data = self.client.query(sql)
            if len(all_data) == 0:
                return []
            return all_data[0]
        except Exception as e:
            self.logging.exception("Exception occurred")
            
        return []


    def user_token(self, user_profile):
        return self.manager.profile_to_embedding(user_profile)


    def _recommend_top(self):
        return self.default_counselor
    

    def _recommend(self, user_embedding):
        D, I = self.index.search(np.array([user_embedding]), self.k)
        counselors = []

        for idx, score in zip(I[0], D[0]):
            # 相似用户uid
            similar_user_id = self.order_user_ids[idx]
            similar_user_counselor = self.order_user_counselor_index.get(similar_user_id, [])

            recommend_data = [{
                'counselor': str(user[0]),
                'score': float(score) * (index + 1),
                'from': 'similar_users {}'.format(similar_user_id),
            } for index, user in enumerate(similar_user_counselor[:self.top_n])]
            counselors.extend(recommend_data)

        counselors.sort(key=lambda x: x['score'])
        return counselors


    def recommend_with_profile(self, user_profile, count=0, is_merge=True):
        user_embedding = self.user_token(user_profile)
        counselors = self._recommend(user_embedding)

        if count > 0:
            counselors.extend(self._recommend_top())

        if is_merge:
            counselor_set = set()
            merged_counselors = []
            for counselor in counselors:
                if counselor['counselor'] not in counselor_set:
                    counselor_set.add(counselor['counselor'])
                    merged_counselors.append(counselor)
            counselors = merged_counselors

        if count > 0:
            counselors = counselors[:count]
        return counselors


    def recommend(self, user_id, count=0, is_merge=True):
        """
        根据用户画像，推荐咨询师
        若获取不到用户画像，推荐默认咨询师（订单最多的）
        """
        user_profile = self.get_user_profile(user_id)
        if not user_profile:
            return self._recommend_top()

        return self.recommend_with_profile(user_profile, count, is_merge)


if __name__ == '__main__':
    recommender = UserCFRecommender()
    print(recommender.recommend('10957910'))